; ---------------------------------------------------------------------------------------------------------
; Simple shell for a simple 6502 computer

; -----------------------------------------------------------
; This function prints prompt and saves the
; starting address

shell_init_line:
  pha

  lda #">"
  jsr vga_set_character
  inc VGA_CURSOR_X

  lda #"_"
  jsr vga_set_character

  ; VGA_ADDRESS stores address of the last written character
  ; Save it to SHELL_CMD_ADDRESS

  lda VGA_ADDRESS
  sta SHELL_CMD_ADDRESS

  lda VGA_ADDRESS + 1
  sta SHELL_CMD_ADDRESS + 1

  pla
  rts


; -----------------------------------------------------------
; Splits input by spaces

shell_process_command:
  pha
  phx
  phy

  stz SHELL_ARGUMENT_COUNT
  
  ; Initialize token value pointer
  lda #<SHELL_TOKEN_VALUES
  sta SHELL_TOKEN_POINTER
  sta SHELL_TOKEN_START
  lda #>SHELL_TOKEN_VALUES
  sta SHELL_TOKEN_POINTER + 1
  sta SHELL_TOKEN_START + 1

  ; Initialize token pointer offset
  lda #<SHELL_TOKEN_POINTERS
  sta SHELL_POINTER_OFFSET
  lda #>SHELL_TOKEN_POINTERS
  sta SHELL_POINTER_OFFSET + 1

  ldy #0
.loop:
  lda (SHELL_CMD_ADDRESS), y
  beq .separator_found        ; null is a line terminator
  iny

  ; Search for a separator
  ldx #0
.separator_loop:
  cmp separators, x
  beq .separator_found

  inx
  cpx #SEPARATOR_COUNT
  beq .no_separator_found     ; End of loop

  jmp .separator_loop

.no_separator_found:
  sta (SHELL_TOKEN_POINTER)
  inc SHELL_TOKEN_POINTER
  jmp .loop

.separator_found:
  pha ; Save character

  ; Save NULL at the end of string
  lda #0
  sta (SHELL_TOKEN_POINTER)
  inc SHELL_TOKEN_POINTER

  ; Save 2 byte token start to pointer array
  lda SHELL_TOKEN_START
  sta (SHELL_POINTER_OFFSET)
  inc SHELL_POINTER_OFFSET

  lda SHELL_TOKEN_START + 1
  sta (SHELL_POINTER_OFFSET)
  inc SHELL_POINTER_OFFSET

  ; Save current TOKEN_POINTER to TOKEN_START
  lda SHELL_TOKEN_POINTER
  sta SHELL_TOKEN_START
  lda SHELL_TOKEN_POINTER + 1
  sta SHELL_TOKEN_START + 1

  inc SHELL_ARGUMENT_COUNT

  pla            ; Restore character
  beq .loop_end  ; If null detected then end

  jmp .loop


.loop_end:
  ; Check the command

  ; Initialize indirect zero-page
  lda SHELL_TOKEN_POINTERS
  sta SHELL_TOKEN_INDIRECT
  lda SHELL_TOKEN_POINTERS + 1
  sta SHELL_TOKEN_INDIRECT + 1

  ; (SHELL_TOKEN_INDIRECT) now points to the first byte of the command string

  ; X - Command index (indicates which command is matched currently)
  ldx #0

.command_loop:
  cpx #CMD_COUNT
  beq .command_not_found

  ; Setup match address
  lda cmd_name_pointers, x
  sta SHELL_MATCH_INDIRECT
  inx

  lda cmd_name_pointers, x
  sta SHELL_MATCH_INDIRECT + 1
  inx

  ; Y - String index (letter being matched)
  ldy #0

.match_loop:
  ; lda (SHELL_TOKEN_INDIRECT), y     ; Load token letter
  ; beq .match_loop_end               ; If null, control reached end of string matching all characters

  ; cmp (SHELL_MATCH_INDIRECT), y     ; Compare to match
  ; bne .command_loop                 ; If no match, then begin another command loop
  ; iny

  lda (SHELL_MATCH_INDIRECT), y       ; Load match character
  cmp (SHELL_TOKEN_INDIRECT), y       ; Compare to token
  bne .command_loop                   ; If not equal, next command

  cmp #0                              ; Compare match to 0 (match ended, token ended)
  beq .match_found                    ; End of loop, match found
  iny

  jmp .match_loop

.match_found:
  txa ; Save X -> A (Jump index)

  ; Match found
  ; Push return address onto stack (reverse order)
  ldx #<(.end-1)
  ldy #>(.end-1)
  phy  ; Push high address
  phx  ; Push low address

  tax ; Restore jump index

  ; Jump to command (-2 since command loop post-increments X)
  jmp (cmd_vectors-2, x) ; This will return directly to .end

.command_not_found:
  ldx #<str_cmd_not_found
  ldy #>str_cmd_not_found
  jsr vga_print_string

.end:
  pla
  ply
  plx

  rts

SEPARATOR_COUNT = 3
separators: .byte " =_" ; Must contain prompt character "_"

CMD_COUNT = 4 * 2 ; Pointers are 2 bit wide

cmd_name_pointers:
  .word cmd_help_name
  .word cmd_set_name
  .word cmd_cls_name
  .word cmd_art_name
cmd_vectors:
  .word cmd_help
  .word cmd_set
  .word cmd_cls
  .word cmd_art

cmd_help_name: .asciiz "help"
cmd_set_name: .asciiz "set"
cmd_cls_name: .asciiz "cls"
cmd_art_name: .asciiz "art"

cmd_help:
  phx
  phy

  ldx #<str_help
  ldy #>str_help
  jsr vga_print_string

  ply
  plx

  rts

str_help: 
  .byte "Help for shell.\r\n"
  .byte "Available commands:\r\n"
  .byte "  help - prints this help,\r\n"
  .byte "  set  - sets variable [set A=1]\r\n"
  .byte "  cls  - clears screen\r\n"
  .byte "  art  - prints ASCII art\r\n\0"

cmd_set:
  pha
  phx
  phy

  lda SHELL_ARGUMENT_COUNT
  cmp #3      ; Check for 3 arguments (command, var name, value)
  bne .usage  ; If not equal to 3, print usage

  ldx #<str_set_setting
  ldy #>str_set_setting
  jsr vga_print_string

  ; Load 2nd argument (var name)
  ldx SHELL_TOKEN_POINTERS + 2
  ldy SHELL_TOKEN_POINTERS + 3
  jsr vga_print_string

  ldx #<str_set_to
  ldy #>str_set_to
  jsr vga_print_string

  ; Load 3rd argument (value)
  ldx SHELL_TOKEN_POINTERS + 4
  ldy SHELL_TOKEN_POINTERS + 5
  jsr vga_print_string

  phx
  phy

  ; Print ($
  ldx #<str_set_hex
  ldy #>str_set_hex
  jsr vga_print_string

  ply
  plx

  ; Print value in hex
  jsr from_dec
  jsr to_hex
  jsr vga_print_string

  ldx #<str_set_end
  ldy #>str_set_end
  jsr vga_print_string

  jmp .end

.usage:
  ldx #<str_set_usage
  ldy #>str_set_usage
  jsr vga_print_string

  jmp .end

.end:
  pla
  ply
  plx

  rts

cmd_cls:
  lda #0
  jsr vga_clear_display

  rts

cmd_art:
  lda SHELL_ARGUMENT_COUNT
  cmp #2

  beq .not_usage
  jmp .usage ; out of range for bne
.not_usage

  ; Load 2nd argument (art name)
  ldx #<str_art_printing
  ldy #>str_art_printing
  jsr vga_print_string

  ; Load 2nd argument (art name)
  ldx SHELL_TOKEN_POINTERS + 2
  ldy SHELL_TOKEN_POINTERS + 3
  jsr vga_print_string
  inc VGA_CURSOR_Y ; newline
  stz VGA_CURSOR_X ; carriage return

  ldy #0
  lda (VGA_STRING_ADDRESS), y
  
  cmp #'1'
  beq .art1
  
  cmp #'2'
  beq .art2

  ; No art
  jmp .usage

.art1:
  ; Clear screen
  lda #0
  jsr vga_clear_display

  ldx #<str_art1
  ldy #>str_art1
  jsr vga_print_string
  ldx #<str_art1_part2
  ldy #>str_art1_part2
  jsr vga_print_string
  ldx #<str_art1_part3
  ldy #>str_art1_part3
  jsr vga_print_string
  ldx #<str_art1_part4
  ldy #>str_art1_part4
  jsr vga_print_string
  ldx #<str_art1_part5
  ldy #>str_art1_part5
  jsr vga_print_string
  ldx #<str_art1_part6
  ldy #>str_art1_part6
  jsr vga_print_string

  jmp .end

.art2:
  ; Clear screen
  lda #0
  jsr vga_clear_display
  ldx #<str_art2_part1
  ldy #>str_art2_part1
  jsr vga_print_string
  ldx #<str_art2_part2
  ldy #>str_art2_part2
  jsr vga_print_string
  ldx #<str_art2_part3
  ldy #>str_art2_part3
  jsr vga_print_string
  ldx #<str_art2_part4
  ldy #>str_art2_part4
  jsr vga_print_string
  jmp .end

.usage:
  ldx #<str_art_usage
  ldy #>str_art_usage
  jsr vga_print_string

.end
  rts

str_cmd_not_found: .asciiz "Command not found.\r\n"

str_set_setting:  .asciiz "Setting "
str_set_to:       .asciiz " to "
str_set_hex:      .asciiz " ($"
str_set_end:      .asciiz ").\r\n"
str_set_usage:    .asciiz "Usage: set [variable]=[value]\r\n"

str_art_printing: .asciiz "Printing art number "
str_art_usage:
  .byte "Command prints ASCII art\r\n\r\n"
  .byte "Available arts:\r\n"
  .byte "1 -- Technika cyfrowa\r\n"
  .byte "2 -- Pink Floyd\r\n"
  .byte 0

str_art1:
  .byte "  _______        _           _ _             \r\n"
  .byte " |__   __|      | |         (_) |            \r\n"
  .byte "    | | ___  ___| |__  _ __  _| | ____ _     \r\n"
  .byte "    | |/ _ \\/ __| '_ \\| '_ \\| | |/ / _` |    \r\n"
  .byte 0
str_art1_part2:
  .byte "    | |  __/ (__| | | | | | | |   < (_| |    \r\n"
  .byte "   _|_|\\___|\\___|_| |_|_| |_|_|_|\\_\\__,_|    \r\n"
  .byte "  / ____|     / _|                           \r\n"
  .byte " | |    _   _| |_ _ __ _____      ____ _     \r\n"
  .byte " | |   | | | |  _| '__/ _ \\ \\ /\\ / / _` |    \r\n"
  .byte 0
str_art1_part3:
  .byte " | |___| |_| | | | | | (_) \\ V  V / (_| |    \r\n"
  .byte "  \\_____\\__, |_| |_|  \\___/ \\_/\\_/ \\__,_|    \r\n"
  .byte " | \\ | | __/ |    | |             | |        \r\n"
  .byte " |  \\| ||___/ _ __| |__   ___ _ __| |_       \r\n"
  .byte " | . ` |/ _ \\| '__| '_ \\ / _ \\ '__| __|      \r\n"
  .byte 0
str_art1_part4:
  .byte " | |\\  | (_) | |  | |_) |  __/ |  | |_       \r\n"
  .byte " |_| \\_|\\___/|_|  |_.__/ \\___|_|   \\__|    _ \r\n"
  .byte " |  \\/  |                            | |  (_)\r\n"
  .byte " | \\  / | ___  _ __ __ ___      _____| | ___ \r\n"
  .byte " | |\\/| |/ _ \\| '__/ _` \\ \\ /\\ / / __| |/ / |\r\n"
  .byte 0
str_art1_part5:
  .byte " | |  | | (_) | | | (_| |\\ V  V /\\__ \\   <| |\r\n"
  .byte " |_|_ |_|\\___/|_|__\\__,_| \\_/\\_/ |___/_|\\_\\_|\r\n"
  .byte " |__ \\ / _ \\__ \\|__ \\                        \r\n"
  .byte "    ) | | | | ) |  ) |                       \r\n"
  .byte "   / /| | | |/ /  / /                        \r\n"
  .byte 0
str_art1_part6:
  .byte "  / /_| |_| / /_ / /_                        \r\n"
  .byte " |____|\\___/____|____|                       \r\n"
  .byte 0

str_art2_part1:
  .byte "        ..uu.\r\n"
  .byte "       ?$\"\"`?i           z'\r\n"
  .byte "       `M  .@\"          x\"\r\n"
  .byte "       'Z :#\"  .   .    f 8M\r\n"
  .byte "       '&H?`  :$f U8   <  MP   x#'\r\n"
  .byte "       d#`    XM  $5.  $  M' xM\"\r\n"
  .byte "     .!\">     @  'f`$L:M  R.@!`\r\n"
  .byte 0
str_art2_part2:
  .byte "    +`  >     R  X  \"NXF  R\"*L\r\n"
  .byte "        k    'f  M   \"$$ :E  5.\r\n"
  .byte "        %    `~  \"    `  'K  'M\r\n"
  .byte "            .uH          'E   `h\r\n"
  .byte "         .x*`             X     `\r\n"
  .byte "      .uf`                *\r\n"
  .byte "    .@8     .\r\n"
  .byte "   'E9F  uf\"          ,     ,\r\n"
  .byte 0
str_art2_part3:
  .byte "     9h+\"   $M    eH. 8b. .8    .....\r\n"
  .byte "    .8`     $'   M 'E  `R;'   d?\"\"\"`\"#\r\n"
  .byte "   ` E      @    b  d   9R    ?*     @\r\n"
  .byte "     >      K.zM `%M'   9'    Xf   .f\r\n"
  .byte "    ;       R'          9     M  .=`\r\n"
  .byte "    t                   M     Mx~\r\n"
  .byte 0
str_art2_part4:
  .byte "    @                  lR    z\"\r\n"
  .byte "    @                  `   ;\"\r\n"
  .byte "                          `\r\n"
  .byte 0