; ---------------------------------------------------------------------------------------------------------
; 6502 library for handling decimals.

sprint_dec:
  ; Converts 16-bit number to it's decimal representation.
  ;
  ; PARAMETERS
  ; X - low byte of the number,
  ; Y - high byte of the number,
  ;
  ; RETURNS
  ; Decimal representation at STRING_ADDRESS - No null termination.

  pha
  phy

  ; dividend = y,x
  stx MODULO_DIVIDEND
  sty MODULO_DIVIDEND+1

  ; divisor = 10
  lda #10
  sta MODULO_DIVISOR
  stz MODULO_DIVISOR+1

  ldy #4  ; End of string

  ; lda #'0'
  ; sta DECIMAL_STRING
  ; sta DECIMAL_STRING+1
  ; sta DECIMAL_STRING+2
  ; sta DECIMAL_STRING+3
  ; sta DECIMAL_STRING+4
  ; stz DECIMAL_STRING+5

.loop:
  jsr divide_with_remainder ; MODULO_DIVIDEND -> (MODULO_DIV_RESULT, MODULO_DIV_REM)
  
  lda MODULO_DIV_REM
  clc
  adc #'0'                  ; Offset digit

  sta DECIMAL_STRING, y
  dey

  lda MODULO_DIV_RESULT
  sta MODULO_DIVIDEND
  lda MODULO_DIV_RESULT+1
  sta MODULO_DIVIDEND+1

  lda MODULO_DIV_RESULT+1
  bne .loop
  lda MODULO_DIV_RESULT
  bne .loop

  ; End loop

  ; phy
  ; ldx #<DECIMAL_STRING
  ; ldy #>DECIMAL_STRING
  ; jsr vga_print_string
  ; inc VGA_CURSOR_Y
  ; stz VGA_CURSOR_X
  ; ply
  
.shift_loop:
  iny
  lda DECIMAL_STRING, y
  sta (STRING_ADDRESS)

  inc STRING_ADDRESS
  bne .no_carry
  inc STRING_ADDRESS+1

.no_carry:
  cpy #4
  bne .shift_loop

.end:
  ; End loop
  ply
  pla
  rts


decimal_mul_10:
  ; Multiply accumulator by 10
  ;
  ; PARAMETERS
  ; A - number to be multiplied
  ;
  ; RETURNS
  ; X - lower byte of the result
  ; Y - higher byte
  
  pha

  stz DECIMAL_MUL10

  clc
  rol a                 ; Multiply accumulator by 2
  rol DECIMAL_MUL10     ; Accept overflow
  sta DECIMAL_MUL10_A2  ; Store A*2

  ldx DECIMAL_MUL10
  phx ; Save overflow on stack

  clc
  rol a
  rol DECIMAL_MUL10
  rol a
  rol DECIMAL_MUL10     ; Multiply A by 4 and accept overflow (A = A*8)

  clc
  adc DECIMAL_MUL10_A2  ; A = A*8 + A*2 = 10 * A
  tax ; Save A10 in X

  pla
  adc DECIMAL_MUL10
  tay ; Y is now overflow

  pla
  rts