; ---------------------------------------------------------------------------------------------------------
; 6502 library for handling dividing with remainder.

divide_with_remainder:
  ; Divides 16-bit number by another 16-bit number
  ; and calculates the remainder.
  ;
  ; PARAMETERS
  ; MODULO_DIVIDEND (2 bytes) - dividend
  ; MODULO_DIVISOR (2 bytes) - divisor
  ;
  ; RETURNS
  ; MODULO_DIV_RESULT (2-bytes) - integer part of the result
  ; MODULO_DIV_REM - remainder

  pha

  stz MODULO_DIV_RESULT
  stz MODULO_DIV_RESULT + 1
  stz MODULO_DIV_REM
  stz MODULO_DIV_REM + 1

  lda #16
  sta MODULO_DIV_SHIFT

.loop:
  ; inc VGA_CURSOR_X

  ; Shift result 1 bit left (*2)
  clc                     ; Clear carry
  rol MODULO_DIV_RESULT   ; Rotate left (C <- M <- 0)
  rol MODULO_DIV_RESULT+1 ; Rotate left (- <- M <- C)

  ; Check if div_rem < divisor (div_rem - divisor < 0 - negative flag set)
  sec                     ; Set carry
  lda MODULO_DIV_REM      ; Load first byte
  sbc MODULO_DIVISOR      ; Subtract first byte
  pha                     ;   Save result (low) on stack
  lda MODULO_DIV_REM+1    ; Load second byte
  sbc MODULO_DIVISOR+1    ; Subtract second byte (perform 2 byte subtraction)
  pha                     ;   Save result (high) on stack
  bcs .one                ; Branch on carry set (+ or 0)

  ; div_rem is less than (10)
  pla
  pla                     ; Remove subtraction result from stack
  ; lda #'0'
  ; jsr vga_set_character
  jmp .end_if

.one:
  ; div_rem was greater than divisor (or equal)
  ; lda #'1'
  ; jsr vga_set_character

  inc MODULO_DIV_RESULT   ; Increment result by 1
  bne .no_carry           ; If not zero, skip carry
  ; If zero, counter overflowed (255+1 = 0)
  inc MODULO_DIV_RESULT+1

.no_carry:
  pla                     ; Pull result (high) from stack
  sta MODULO_DIV_REM+1
  pla                     ; Pull result (low) from stack
  sta MODULO_DIV_REM

.end_if:
  lda MODULO_DIV_SHIFT
  beq .loop_end           ; End loop if shift is 0

  ; Shift div_rem 1 bit left (*2)
  clc
  rol MODULO_DIVIDEND
  rol MODULO_DIVIDEND+1
  rol MODULO_DIV_REM
  rol MODULO_DIV_REM+1

  dec MODULO_DIV_SHIFT

  jmp .loop

.loop_end:
  ; inc VGA_CURSOR_X
  pla
  rts