; ---------------------------------------------------------------------------------------------------------
; 6502 library for handling small (16-bit) floating point digits

float_load_1:
  ; Loads float number into:
  ;  FLOAT_MANTISSA1
  ;  FLOAT_EXPO_1
  ;  FLOAT_SIGN_1
  ;
  ; PARAMETERS
  ; X - low byte of the number,
  ; Y - high byte of the number.

  pha

  stx FLOAT_MANTISSA1+2  ; Store low byte to 3B
  sty FLOAT_MANTISSA1+1  ; Store high byte to 2B
  stz FLOAT_MANTISSA1+3  ; Clear 4B

  ror FLOAT_MANTISSA1+1  ; Rotate right (/2) - -> 2B
  ror FLOAT_MANTISSA1+2  ; Rotate right (/2) 2B -> 3B
  ror FLOAT_MANTISSA1+3  ; Rotate right (/2) 3B -> 4B
  ror FLOAT_MANTISSA1+1
  ror FLOAT_MANTISSA1+2
  ror FLOAT_MANTISSA1+3  ; 2 binary places

  lda FLOAT_MANTISSA1+1  ; Load Sign + Expo
  and #$1F              ; Isolate expo
  sta FLOAT_EXPO1

  lda FLOAT_MANTISSA1+1  ; Load Sign + Expo
  and #$20              ; Isolate sign (as $20 or $00)
  sta FLOAT_SIGN1

  ; Store leading 1.
  lda #1
  sta FLOAT_MANTISSA1+1

  ; Assign 0 for the rest
  stz FLOAT_MANTISSA1+0
  stz FLOAT_MANTISSA1+4

  pla
  rts

float_load_2:
  ; Loads float number into:
  ;  FLOAT_MANTISSA2
  ;  FLOAT_EXPO2
  ;  FLOAT_SIGN2
  ;
  ; PARAMETERS
  ; X - low byte of the number,
  ; Y - high byte of the number.

  pha

  stx FLOAT_MANTISSA2+2  ; Store low byte to 3B
  sty FLOAT_MANTISSA2+1  ; Store high byte to 2B
  stz FLOAT_MANTISSA2+3  ; Clear 4B

  ror FLOAT_MANTISSA2+1  ; Rotate right (/2) - -> 2B
  ror FLOAT_MANTISSA2+2  ; Rotate right (/2) 2B -> 3B
  ror FLOAT_MANTISSA2+3  ; Rotate right (/2) 3B -> 4B
  ror FLOAT_MANTISSA2+1
  ror FLOAT_MANTISSA2+2
  ror FLOAT_MANTISSA2+3  ; 2 binary places

  lda FLOAT_MANTISSA2+1  ; Load Sign + Expo
  and #$1F              ; Isolate expo
  sta FLOAT_EXPO2

  lda FLOAT_MANTISSA2+1  ; Load Sign + Expo
  and #$20              ; Isolate sign (as $20 or $00)
  sta FLOAT_SIGN2

  ; Store leading 1.
  lda #1
  sta FLOAT_MANTISSA2+1

  ; Assign 0 for the rest
  stz FLOAT_MANTISSA2+0
  stz FLOAT_MANTISSA2+4

  pla
  rts

float_rotate_1:
  ; Rotate FLOAT_MANTISSA1 A-bits to the
  ; X=1 - right (negative expo)
  ; X=0 - left (positive expo)
  ;
  ; PARAMETERS
  ; A - number of rotations
  ; X - direction

  pha

  cmp #0
  beq .end

  cpx #0
  beq .rotate_left

  ; Rotate right - negative expo
  clc
.loop:
  ror FLOAT_MANTISSA1+1
  ror FLOAT_MANTISSA1+2
  ror FLOAT_MANTISSA1+3
  ror FLOAT_MANTISSA1+4
  dea
  bne .loop
  jmp .end

.rotate_left:
  ; Rotate left - positive expo
  clc
.rotate_left_loop:
  rol FLOAT_MANTISSA1+3
  rol FLOAT_MANTISSA1+2
  rol FLOAT_MANTISSA1+1
  rol FLOAT_MANTISSA1+0
  dea
  bne .rotate_left_loop

.end:
  pla
  rts

float_rotate_2:
  ; Rotate FLOAT_MANTISSA2 A-bits to the
  ; X=1 - right (negative expo)
  ; X=0 - left (positive expo)
  ;
  ; PARAMETERS
  ; A - number of rotations
  ; X - direction

  pha

  cmp #0
  beq .end

  cpx #0
  beq .rotate_left

  ; Rotate right - negative expo
  clc
.loop:
  ror FLOAT_MANTISSA2+1
  ror FLOAT_MANTISSA2+2
  ror FLOAT_MANTISSA2+3
  ror FLOAT_MANTISSA2+4
  dea
  bne .loop
  jmp .end

.rotate_left:
  ; Rotate left - positive expo
  clc
.rotate_left_loop:
  rol FLOAT_MANTISSA2+3
  rol FLOAT_MANTISSA2+2
  rol FLOAT_MANTISSA2+1
  rol FLOAT_MANTISSA2+0
  dea
  bne .rotate_left_loop

.end:
  pla
  rts

float_save_2:
  ; Saves float number from:
  ;  FLOAT_MANTISSA2
  ;  FLOAT_EXPO2
  ;  FLOAT_SIGN2
  ;
  ; RETURNS
  ; X - low byte of the number,
  ; Y - high byte of the number.

  pha

  lda FLOAT_EXPO2
  ldx FLOAT_SIGN2
  beq .sign_0
  ; Sign 1
  ora #%00100000
.sign_0:

  ; Save Sign + Expo to 2B
  sta FLOAT_MANTISSA2+1
  ldx #0
  lda #2
  jsr float_rotate_2

  ldx FLOAT_MANTISSA2+2
  ldy FLOAT_MANTISSA2+1

  pla
  rts

sprint_float:
  ; Converts 16-bit float number to it's decimal representation.
  ;
  ; PARAMETERS
  ; X - low byte of the number,
  ; Y - high byte of the number,
  ;
  ; RETURNS
  ; Decimal representation at STRING_ADDRESS - No null termination.

  pha

  ; Load the float to set 1
  jsr float_load_1

  lda FLOAT_EXPO1
  sec
  sbc #15   ; -15 (bias)
  bpl .positive_expo

  ; Negative expo
  eor #$FF  ; Negate
  clc
  adc #1    ; Add +1 (U2)

  ldx #1    ; ROR
  jmp .rotate

.positive_expo:
  ldx #0

.rotate:
  jsr float_rotate_1

  lda FLOAT_SIGN1
  beq .print_integer
  
  ; Sign non-zero
  lda #'-'
  sta (STRING_ADDRESS)
 
  inc STRING_ADDRESS
  bne .print_integer
  inc STRING_ADDRESS+1

.print_integer:
  ldx FLOAT_MANTISSA1+1
  ldy FLOAT_MANTISSA1+0
  jsr sprint_dec

  lda #'.'
  sta (STRING_ADDRESS)
 
  inc STRING_ADDRESS
  bne .fraction_loop
  inc STRING_ADDRESS+1

.fraction_loop:
  stz FLOAT_MANTISSA1+1
  stz FLOAT_MANTISSA1+0
  stz FLOAT_OVF
  
  ldx #4
.loop:
  phx
  lda FLOAT_MANTISSA1, x
  jsr decimal_mul_10
  
  txa           ; A = X = low
  clc
  adc FLOAT_OVF ; A = X +OVF
  plx
  sta FLOAT_MANTISSA1, x
  sty FLOAT_OVF

  dex
  bne .loop

  ; End loop
  ldx FLOAT_MANTISSA1+1
  ldy FLOAT_MANTISSA1+0
  jsr sprint_dec

  lda FLOAT_MANTISSA1+2
  bne .fraction_loop
  lda FLOAT_MANTISSA1+3
  bne .fraction_loop
  lda FLOAT_MANTISSA1+4
  bne .fraction_loop

.end:
  pla
  rts



float_denormalize:
  ; Denormalizes 2 loaded float numbers to the average
  ; of their exponents
  ;
  ; PARAMETERS
  ; 2 loaded float numbers
  ;
  ; RETURNS
  ; 2 denormalized float numbers
  ; FLOAT_AVG - current exponent

  pha
  phx

  lda FLOAT_EXPO1   ; Load e1
  clc
  adc FLOAT_EXPO2   ; Add e2
  lsr               ; Divide by 2 - average
  sta FLOAT_AVG     ; Store average in FLOAT_AVG

  ; Float 1
  lda FLOAT_EXPO1
  sec
  sbc FLOAT_AVG
  bpl .float1_positive
  ; Float 1 - negative shift
  ; U2
  eor #$FF
  clc
  adc #1
  ldx #1
  jmp .float1_rotate

.float1_positive:
  ldx #0

.float1_rotate:
  jsr float_rotate_1

  ; Float 2
  lda FLOAT_EXPO2
  sec
  sbc FLOAT_AVG
  bpl .float2_positive
  ; Float 2 - negative shift
  ; U2
  eor #$FF
  clc
  adc #1
  ldx #1
  jmp .float2_rotate

.float2_positive:
  ldx #0

.float2_rotate:
  jsr float_rotate_2

  plx
  pla
  rts


float_normalize_2:
  ; Normalizes 2 float numbers
  ;
  ; PARAMETERS
  ; 2 loaded float numbers
  ;
  ; RETURNS
  ; 2 normalized float numbers
  ; FLOAT_AVG - current exponent

  pha
  phx

.loop:
  lda FLOAT_MANTISSA2+1
  cmp #1                  ; Compare 2B with 1
  beq .end
  bpl .right
  ; B2<1 - shift left
  ldx #0
  dec FLOAT_AVG
  jmp .shift

.right:
  ldx #1
  inc FLOAT_AVG

.shift:
  lda #1
  jsr float_rotate_2
  jmp .loop

.end:
  plx
  pla
  rts


float_add:
  ; Adds two floats
  ;
  ; PARAMETERS
  ; 2 floats x 2 bytes on pStack
  ;
  ; RETURNS
  ; One float on pStack

  pha
  phy

  jsr pstack_pull
  jsr float_load_1
  jsr pstack_pull
  jsr float_load_2
  ; 2 floats loaded

  jsr float_denormalize
  ; Denormalized

  lda FLOAT_SIGN1
  cmp FLOAT_SIGN2
  beq .same_sign
  
  ; Different signs - subtraction
  sec
  ldy #4
.sub_loop:
  lda FLOAT_MANTISSA1, y
  sbc FLOAT_MANTISSA2, y
  sta FLOAT_MANTISSA2, y
  dey
  bpl .sub_loop
  ; Float2 = Float1 - Float2

  ; If carry set - no borrow -> Float1 > Float2 -> Sign = Sign1
  ; If carry clear - borrow  -> Float2 > Float1 -> Sign = Sign2
  bcc .sub_end
  ; Set sign to Sign1
  lda FLOAT_SIGN1
  sta FLOAT_SIGN2

.sub_end:
  jmp .end

.same_sign
  ; Adding 2 floats with the same sign
  clc
  ldy #4
.add_loop:
  lda FLOAT_MANTISSA1, y
  adc FLOAT_MANTISSA2, y
  sta FLOAT_MANTISSA2, y
  dey
  bpl .add_loop
  ; Float2 = Float1 + Float2

.end:
  jsr float_normalize_2

  lda FLOAT_AVG
  sta FLOAT_EXPO2

  jsr float_save_2
  jsr pstack_push

  ply
  pla
  rts