sprintf:
  ; Formats a string and saves it to STRING_ADDRESS location.
  ;
  ; PARAMETERS
  ; X - low-byte address of a format string
  ; Y - high-byte
  ; pStack (Parameter stack, passed in little-endian, 16-bits)
  ;
  ; SUPPORTED FORMATTING
  ; %d - Integer                (16-bit)
  ; %D - Signed integer         ( 8-bit)
  ; %X - Hexadecimal  integer   (16-bit)
  ; %x - Hexadecimal  integer   ( 8-bit)
  ; %f - Small float            (16-bit)
  ;
  ; RETURNS
  ; Formatted string at STRING_ADDRESS - Null terminated.

  pha
  phy

  stx FORMAT_STRING_ADDRESS
  sty FORMAT_STRING_ADDRESS+1

  ldy #0  ; Format string index

.loop:
  lda (FORMAT_STRING_ADDRESS), y
  beq .end            ; If loaded null -> end
  iny

  cmp #'%'
  beq .format         ; If format delimeter -> format

  sta (STRING_ADDRESS)

.loop_finish:
  inc STRING_ADDRESS
  bne .no_carry
  inc STRING_ADDRESS+1
.no_carry:

  jmp .loop

.format:
  ; Load format
  lda (FORMAT_STRING_ADDRESS), y
  iny
  phy

  cmp #'d'
  beq .format_decimal     ; Unsigned 16-bit
  cmp #'D'
  beq .format_decimal_u2  ; Signed 8-bit
  
  cmp #'X'
  beq .format_hex         ; 16-bit
  cmp #'x'
  beq .format_hex_8       ; 8-bit
  
  cmp #'f'
  beq .format_float

  cmp #'%'
  beq .print_percent

.format_loop_finish:
  ply
  jmp .loop


.format_decimal:
  jsr pstack_pull
  jsr sprint_dec
  jmp .format_loop_finish


.format_decimal_u2:
  jsr pstack_pull
  ldy #0
  
  txa
  bpl .positive
  ; Negative
  eor #$FF  ; Negate
  clc
  adc #1    ; Add +1 (U2)
  tax

  lda #"-"
  sta (STRING_ADDRESS)
  inc STRING_ADDRESS
  bne .positive
  inc STRING_ADDRESS+1

.positive:
  jsr sprint_dec
  jmp .format_loop_finish


.format_hex:
  jsr pstack_pull
  lda #2
  jsr sprint_hex
  jmp .format_loop_finish

.format_hex_8:
  jsr pstack_pull_8
  lda #1
  jsr sprint_hex
  jmp .format_loop_finish


.format_float:
  jsr pstack_pull
  jsr sprint_float
  jmp .format_loop_finish


.print_percent:
  sta (STRING_ADDRESS)
  jmp .loop_finish


.end:
  lda #0
  sta (STRING_ADDRESS)

  ply
  pla
  rts



printf:
  ; Same as above but prepares STRING_ADDRESS
  ; And X,Y registers to call *_print_string

  pha

  lda #<STRING_FORMATTED
  sta STRING_ADDRESS 
  lda #>STRING_FORMATTED
  sta STRING_ADDRESS+1

  jsr sprintf

  ldx #<STRING_FORMATTED
  ldy #>STRING_FORMATTED

  pla
  rts