; ---------------------------------------------------------------------------------------------------------
; PS/2 homemade keyboard controller

; PS/2 registers
PS2_STATUS_REGISTER = $6D00
PS2_DATA_REGISTER   = $6D01

; PS/2 status masks
PS2_INT_MASK      = %10000000
PS2_BREAK_MASK    = %01000000
PS2_EXTENDED_MASK = %00100000

; PS/2 modifier masks (PS2_MODIFIERS)
PS2_SHIFT_MASK = %10000000

ps2_interrupt_handler:
  pha
  phx

  ldx PS2_DATA_REGISTER     ; Load scan code
  lda PS2_STATUS
  jsr lcd_home
  jsr lcd_print_debug
  cpx #$66                  ; Backspace
  beq .backspace

  cpx #$12                  ; Left shift
  beq .shift
  cpx #$59                  ; Right shift
  beq .shift

  cpx #$5A                  ; Enter
  beq .enter

  ; Not backspace, not shift, MAKE character
  lda PS2_STATUS            ; Load status
  and #PS2_BREAK_MASK       ; Check for BREAK
  bne .end                  ; If break, end

  lda VGA_CURSOR_X          ; Load cursor column
  cmp #VGA_COLS-1           ; Check if the cursor is in the last column
  beq .end                  ; If so, end. 

  ; Check for shift
  lda PS2_MODIFIERS
  and #PS2_SHIFT_MASK
  beq .print_char

  ; Shift pressed
  txa               ; X -> A
  ora #$80          ; Shift character into next half-byte
  tax               ; A -> X        

.print_char:
  lda scancodes, x
  jsr vga_set_character

  inc VGA_CURSOR_X
  lda #"_"
  jsr vga_set_character

  jmp .end

.backspace:
  lda PS2_STATUS            ; Load status
  and #PS2_BREAK_MASK       ; Check for BREAK
  bne .end                  ; If break, end

  lda VGA_CURSOR_X          ; Load cursor column
  cmp #1                    ; Check if the cursor is in the 2nd column
  beq .end                  ; If so, end.

  ; Clear current underscore
  lda #0
  jsr vga_set_character

  ; Set new one
  dec VGA_CURSOR_X
  lda #"_"
  jsr vga_set_character

  jmp .end

.shift:
  lda PS2_STATUS            ; Load status
  and #PS2_BREAK_MASK       ; Check for BREAK
  bne .shift_break          ; If break, end

  ; Shift make
  lda PS2_MODIFIERS
  ora #PS2_SHIFT_MASK
  sta PS2_MODIFIERS
  jmp .end

.shift_break
  lda PS2_MODIFIERS
  and #(~PS2_SHIFT_MASK & $FF)
  sta PS2_MODIFIERS
  jmp .end

.enter:
  lda PS2_STATUS            ; Load status
  and #PS2_BREAK_MASK       ; Check for BREAK
  bne .end                  ; If break, end

  ; Clear underscore
  lda #0
  jsr vga_set_character

  ; New line
  stz VGA_CURSOR_X
  inc VGA_CURSOR_Y

  jsr shell_process_command
  jsr shell_init_line

  jmp .end
  
.end:
  plx
  pla

  rts


scancodes:
  .byte 0,0,0,0,0,0,0,0,0,0,0,0,0,0,'`',0,0,0,0,0,0,'q','1',0,0,0,'z','s','a','w','2',0,0,'c','x','d','e','4','3',0,0,' ','v','f','t','r','5',0,0,'n','b','h','g','y','6',0,0,0,'m','j','u','7','8',0,0,',','k','i','o','0','9',0,0,'.','/','l',';','p','-',0,0,0,"'",0,'[','=',0,0,0,0,0,']',0,'\\',0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,'~',0,0,0,0,0,0,'Q','!',0,0,0,'Z','S','A','W','@',0,0,'C','X','D','E','$','#',0,0,' ','V','F','T','R','%',0,0,'N','B','H','G','Y','^',0,0,0,'M','J','U','&','*',0,0,'<','K','I','O',')','(',0,0,'>','?','L',':','P','_',0,0,0,'"',0,'{','+',0,0,0,0,0,'}',0,'|',0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
