; ---------------------------------------------------------------------------------------------------------
; 6502 library for handling parameter stack.

pstack_init:
  ; Takes care of initializing the pstack

  pha

  ; Init pointer
  lda #<PSTACK_START
  sta PSTACK_POINTER
  lda #>PSTACK_START
  sta PSTACK_POINTER+1

  pla
  rts

pstack_push:
  ; Pushes 16-bit value onto the stack
  ;
  ; PARAMETERS
  ; X - low byte of the value
  ; Y - high byte

  pha

  txa                     ; X -> A (low-byte)
  sta (PSTACK_POINTER)    ; Save at pointer

  inc PSTACK_POINTER
  tya                     ; Y -> A (high-byte)
  sta (PSTACK_POINTER)    ; Save at pointer

  inc PSTACK_POINTER

  pla
  rts

pstack_push_8:
  ; Pushes 8-bit value onto the stack
  ;
  ; PARAMETERS
  ; X - low byte of the value

  pha

  txa                     ; X -> A (low-byte)
  sta (PSTACK_POINTER)    ; Save at pointer
  inc PSTACK_POINTER

  pla
  rts

pstack_pull:
  ; Pulls 16-bit value onto the stack
  ;
  ; RETURNS
  ; X - low byte of the value
  ; Y - high byte

  pha

  dec PSTACK_POINTER
  lda (PSTACK_POINTER)    ; Read from pointer
  tay                     ; A -> Y (high-byte)

  dec PSTACK_POINTER
  lda (PSTACK_POINTER)    ; Read from pointer
  tax                     ; A -> X (low-byte)

  pla
  rts

pstack_pull_8:
  ; Pulls 8-bit value onto the stack
  ;
  ; RETURNS
  ; X - low byte of the value

  pha

  dec PSTACK_POINTER
  lda (PSTACK_POINTER)    ; Read from pointer
  tax                     ; A -> X (low-byte)

  pla
  rts