; ---------------------------------------------------------------------------------------------------------
; Custom-made VGA graphics support

VGA_BASE_ADDRESS = $7000
VGA_ROWS = 38
VGA_COLS = 50

vga_print_string:
  ; Prints string at current position (VGA_CURSOR_X/Y) onto the
  ; VGA display.
  ;
  ; PARAMETERS
  ; X - Low byte address of the string
  ; Y - High byte

  stx VGA_STRING_ADDRESS
  sty VGA_STRING_ADDRESS + 1

  ldy #0
  pha
  
.loop:
  lda (VGA_STRING_ADDRESS), y
  beq .end  ; If null, then end
  iny

  cmp #"\n"
  beq .new_line

  cmp #"\r"
  beq .carriage_return

  ; Else print character
  jsr vga_set_character
  inc VGA_CURSOR_X
  jmp .loop

.new_line:
  inc VGA_CURSOR_Y
  jmp .loop

.carriage_return:
  stz VGA_CURSOR_X
  jmp .loop

.end:
  ldx VGA_STRING_ADDRESS
  ldy VGA_STRING_ADDRESS + 1
  pla
  
  rts

vga_clear_display:
  ; PARAMETERS
  ; A - character

  phx
  phy

  ldy #0

.row_loop:
  sty VGA_CURSOR_Y        ; Store row no
  ldx #0                  ; Reset column counter

.column_loop:
  stx VGA_CURSOR_X        ; Store column no
  jsr vga_set_character

  inx                     ; Increment column no
  cpx #VGA_COLS           ; Compare with total columns
  bne .column_loop        ; while not equal repeat

  iny                     ; Increment row no
  cpy #VGA_ROWS           ; Compare with total rows
  bne .row_loop           ; while not equal repeat

  ply
  plx

  stz VGA_CURSOR_X
  stz VGA_CURSOR_Y

  rts

vga_set_character:
  ; PARAMETERS
  ; VGA_CURSOR_X - column
  ; VGA_CURSOR_Y - row
  ; Accumulator  - character
  ;
  ; VGA_ADDRESS stores address of the last written character

  pha

  ; Calculate low byte address
  lda VGA_CURSOR_Y  ; Load row
  asl
  asl
  asl
  asl
  asl
  asl               ; Shift 6 bits to left
  ora VGA_CURSOR_X  ; OR with column index
  sta VGA_ADDRESS   ; A -> MEM(low byte)

  ; Calculate low high address
  lda VGA_CURSOR_Y  ; Load row
  lsr
  lsr               ; Shift 2 bits right
  ora #$70          ; OR with VGA address
  sta VGA_ADDRESS+1 ; A -> MEM(high byte)
  
  pla
  sta (VGA_ADDRESS)

  rts