; ------------------------------------------------------------
; Library for number conversions

mul_10:
  ; Multiply number in A by 10
  ; A = A*2 + A*8 = A*10

  asl             ; A = A*2
  sta NUMBER_A2   ; Save A*2 to MEM
  
  asl
  asl             ; A = A*8

  clc
  adc NUMBER_A2   ; A = A*8 + MEM(A*2)

  rts

from_dec:
  ; Convert decimal number and store it in A
  ;
  ; PARAMETERS
  ; X - low address of the decimal string
  ; Y - high address of the decimal string
  ;
  ; RETURNS
  ; A - value of the decimal string

  stx NUMBER_STRING_ADDRESS
  sty NUMBER_STRING_ADDRESS + 1

  ; Clear result
  stz NUMBER_STR_RESULT

  jsr string_length     ; A = string length

  tay     ; Y = length of string
  dey     ; Y = length of string - 1
  
  ldx #0  ; current power of 10

.loop:
  lda (NUMBER_STRING_ADDRESS), y
  dey

  sec            ; Set carry
  sbc #"0"       ; A = A - '0' (A now 0-9)

  phx

.mul_loop:
  cpx #0
  beq .mul_loop_end

  jsr mul_10
  dex

  jmp .mul_loop

.mul_loop_end:
  ; Restore X from stack and increment
  plx
  inx

  ; Add A to result and store it
  clc
  adc NUMBER_STR_RESULT
  sta NUMBER_STR_RESULT

  cpy #$FF ; Underflow to FF
  bne .loop

.end:
  ldx NUMBER_STRING_ADDRESS
  ldy NUMBER_STRING_ADDRESS + 1
  lda NUMBER_STR_RESULT

  rts

to_hex:
  ; Converts content of A register into a hex string
  ; stored at NUMBER_HEX address. The address is autosaved to X, Y for print
  ;
  ; PARAMETERS
  ; A - value to convert
  ;
  ; RETURNS
  ; X - Low part of the address of the result
  ; Y - High part of the address of the result
  ; 
  ; EXAMPLE:
  ; lda #55
  ; jsr to_hex
  ; jsr lcd_print_string

  pha         ; Save A to stack twice 
  pha         ; (one for conversion, other one for retrieval)
  ldx #1      ; Load index 

  ; Low nibble
  and #$0F
  bra .save

.second:
  pla         ; Pull A from stack
  lsr
  lsr
  lsr
  lsr         ; A now contains the high nibble only
  dex         ; Decrement character index

.save:
  tay
  lda charset, y        ; Load offsetted character
  sta NUMBER_HEX, x     ; Save it at indexed position

  cpx #0
  bne .second            ; If X is not 0, go to second digit

  stz NUMBER_HEX+2

  ldx #<NUMBER_HEX
  ldy #>NUMBER_HEX

  pla
  rts

charset:
  .ascii "0123456789ABCDEF"