; ---------------------------------------------------------------------------------------------------------
; ACIA 65c55-based UART support

ACIA_DATA_REG = $6E00
ACIA_STATUS_REG = $6E01
ACIA_COMMAND_REG = $6E02
ACIA_CONTROL_REG = $6E03

ACIA_STATUS_RX_FULL = %00001000
ACIA_STATUS_TX_EMPTY = %00010000

ACIA_COMMAND_ENABLE = %00000001
ACIA_COMMAND_TX     = %00001000

ACIA_CONTROL_BAUD_RATE = %00010000
ACIA_CONTROL_100b      = %00000001
ACIA_CONTROL_9600b     = %00001100
ACIA_CONTROL_38400b    = %00001111

WAIT_CYCLES_260us      = 1


uart_init:
  ; Inits the serial communication

  lda #(ACIA_CONTROL_BAUD_RATE | ACIA_CONTROL_38400b)
  sta ACIA_CONTROL_REG

  lda #(ACIA_COMMAND_ENABLE | ACIA_COMMAND_TX)
  sta ACIA_COMMAND_REG

  rts

uart_recv:
  ; Receives a byte
  ;
  ; RETURNS
  ; A - Data received

  lda ACIA_STATUS_REG
  and #ACIA_STATUS_RX_FULL
  beq uart_recv                  ; Loop waiting for character

  lda ACIA_DATA_REG
  rts
  

uart_recv_with_timeout:
  ; Receives a byte with a timeout
  ;
  ; PARAMETERS
  ; Y - try count (each try ~10ms)
  ;
  ; RETURNS
  ; Y - (0 - success, FF - failed)
  ; A - data
  ;
  ; Overhead
  ; jsr - 6 cycles
  ; rts - 6 cycles
  ; 12 overhead
  ; 25 loop

.loop:
  lda ACIA_STATUS_REG         ; 4 cycles
  and #ACIA_STATUS_RX_FULL    ; 2 cycles
  bne .success                ; 2 cycles

  ; Read failed (no data to read)
  ; Wait 10ms
  lda #73
  jsr delay_a

  dey
  bne .loop
  ; End loop

  ; Timeout
  ldy #$FF
  jmp .end

.success:
  ; Read successful
  lda ACIA_DATA_REG
  ldy #0

.end
  rts
  

uart_send:
  ; Sends a byte
  ;
  ; A - Data to be send

  sta ACIA_DATA_REG

  pha                           ; 3 cycles
  lda #WAIT_CYCLES_260us        ; 2 cycles
  jsr delay_a                   ; - cycles
  pla                           ; 4 cycles
  ;                             ; 9 cycles

  rts


uart_print_string:
  ; PARAMETERS
  ; X, Y - Address of the string (X - Low, Y - High byte)

  stx UART_STRING_ADDRESS
  sty UART_STRING_ADDRESS + 1

  ldy #0
  pha
  
.loop:
  lda (UART_STRING_ADDRESS), y
  beq .end  ; If null, then end
  iny

  ; Else send character
  jsr uart_send
  jmp .loop

.end:
  pla
  rts