; ---------------------------------------------------------------------------
; Zero-page variables
LCD_STRING_ADDRESS  = $00 ; 2 bytes
BTLDR_PAGE_ADDRESS  = $02 ; 2 bytes

VGA_ADDRESS  = $04 ; 2 bytes
VGA_CURSOR_X = $06 ; 1 byte
VGA_CURSOR_Y = $07 ; 1 byte
VGA_STRING_ADDRESS   = $08 ; 2 bytes
UART_STRING_ADDRESS  = $0A ; 2 bytes
SHELL_CMD_ADDRESS    = $0C ; 2 bytes
SHELL_TOKEN_POINTER  = $0E ; 2 bytes - used to access SHELL_TOKEN_VALUES
SHELL_POINTER_OFFSET = $10 ; 2 bytes - used to access SHELL_TOKEN_POINTERS
SHELL_TOKEN_INDIRECT = $12 ; 2 bytes
SHELL_MATCH_INDIRECT = $14 ; 2 bytes

STRING_ADDRESS          = $30 ; 2 bytes
NUMBER_STRING_ADDRESS   = $32 ; 2 bytes
FORMAT_STRING_ADDRESS   = $34 ; 2 bytes
PSTACK_POINTER          = $36 ; 2 bytes - Pointer to parameter stack

; ---------------------------------------------------------------------------
; Normal variables (page 2)
LCD_NIBBLE     = $0200 ; 1 byte
LCD_LINE       = $0201 ; 1 byte

NUMBER_HEX        = $0202 ; 3 byte
NUMBER_DIGIT      = $0205 ; 1 byte
NUMBER_A2         = $0206 ; 1 byte
NUMBER_STR_LEN    = $0207 ; 1 byte
NUMBER_STR_RESULT = $0208 ; 1 byte

PS2_MODIFIERS  = $0210 ; 1 byte
PS2_STATUS     = $0211 ; 1 byte

HEX_BYTE_COUNT      = $021D ; 1 bytes - hex byte count
HEX_NUMBER          = $021E ; 2 bytes - hex representation
DECIMAL_MUL10       = $0220 ; 1 byte - overflowed mul 10 result
DECIMAL_MUL10_A2    = $0221 ; 1 byte - A*2
DECIMAL_STRING      = $0230 ; 5 bytes - decimal representation

; ---------------------------------------------------------------------------
; Shell variables (page 3,4)
SHELL_TOKEN_VALUES   = $0300 ; 64 bytes (16 tokens max * 4 characters)
SHELL_TOKEN_POINTERS = $0340 ; 32 bytes (16 2-byte token pointers)
SHELL_TOKEN_START    = $0360 ; 2 bytes
SHELL_ARGUMENT_COUNT = $0462 ; 1 byte

; ---------------------------------------------------------------------------
; Math variabless (page 7)
;  Multiplying, dividing, modulo, floating point
MODULO_DIVIDEND     = $0700 ; 2 bytes (dividend passed to div_10_remainder)
MODULO_DIVISOR      = $0702 ; 2 bytes (divisor passed to div_10_remainder)
MODULO_DIV_RESULT   = $0704 ; 2 bytes (result from div_10_remainder)
MODULO_DIV_REM      = $0706 ; 2 bytes (remainder from div_10_remainder)
MODULO_DIV_SHIFT    = $0708 ; 1 byte (number of shifts)

FLOAT_MANTISSA1     = $0709 ; 5 bytes (40-bits)
FLOAT_MANTISSA2     = $070E ; 5 bytes
FLOAT_EXPO1         = $0713 ; 1 byte
FLOAT_EXPO2         = $0714 ; 1 byte
FLOAT_SIGN1         = $0715 ; 1 byte
FLOAT_SIGN2         = $0716 ; 1 byte
FLOAT_OVF           = $0717 ; 1 byte 
FLOAT_AVG           = $0718 ; 1 byte - average of exponents


; ---------------------------------------------------------------------------
; Formatted printing variables (page 8)

STRING_FORMATTED    = $0800 ; 128 bytes - Result of the formatting

PSTACK_START        = $08E0 ; 32 bytes - Parameter stack